Class: Toys::Completion::Candidate
- Inherits:
-
Object
- Object
- Toys::Completion::Candidate
- Includes:
- Comparable
- Defined in:
- lib/toys/completion.rb
Overview
A candidate for completing a string fragment.
A candidate includes a string representing the potential completed word, as well as a flag indicating whether it is a partial completion (i.e. a prefix that could still be added to) versus a final word. Generally, tab completion systems should add a trailing space after a final completion but not after a partial completion.
Instance Attribute Summary collapse
-
#string ⇒ String
(also: #to_s)
readonly
Get the candidate string.
Class Method Summary collapse
-
.new_multi(array, partial: false) ⇒ Array<Toys::Completion::Candidate>
Create an array of candidates given an array of strings.
Instance Method Summary collapse
-
#final? ⇒ boolean
Determine whether the candidate is a final completion.
-
#initialize(string, partial: false) ⇒ Candidate
constructor
Create a new candidate.
-
#partial? ⇒ boolean
Determine whether the candidate is partial completion.
Constructor Details
#initialize(string, partial: false) ⇒ Candidate
Create a new candidate
160 161 162 163 |
# File 'lib/toys/completion.rb', line 160 def initialize(string, partial: false) @string = string.to_s @partial = partial ? true : false end |
Instance Attribute Details
#string ⇒ String (readonly) Also known as: to_s
Get the candidate string.
169 170 171 |
# File 'lib/toys/completion.rb', line 169 def string @string end |
Class Method Details
.new_multi(array, partial: false) ⇒ Array<Toys::Completion::Candidate>
Create an array of candidates given an array of strings.
215 216 217 |
# File 'lib/toys/completion.rb', line 215 def self.new_multi(array, partial: false) array.map { |s| new(s, partial: partial) } end |
Instance Method Details
#final? ⇒ boolean
Determine whether the candidate is a final completion.
184 185 186 |
# File 'lib/toys/completion.rb', line 184 def final? !@partial end |
#partial? ⇒ boolean
Determine whether the candidate is partial completion.
176 177 178 |
# File 'lib/toys/completion.rb', line 176 def partial? @partial end |