Class: Toys::Completion::Context
- Inherits:
-
Object
- Object
- Toys::Completion::Context
- Defined in:
- lib/toys/completion.rb
Overview
The context in which to determine completion candidates.
Instance Attribute Summary collapse
-
#cli ⇒ Toys::CLI
readonly
The CLI being run.
-
#fragment ⇒ String
readonly
The current string fragment to complete.
-
#fragment_prefix ⇒ String
readonly
A non-completed prefix for the current fragment.
-
#previous_words ⇒ Array<String>
readonly
All previous words.
Instance Method Summary collapse
-
#[](key) ⇒ Object
(also: #get)
Get data for arbitrary key.
-
#arg_parser ⇒ Toys::ArgParser
Current ArgParser indicating the status of argument parsing up to this point.
-
#args ⇒ Array<String>
An array of complete arguments passed to the tool, prior to the fragment to complete.
-
#initialize(cli:, previous_words: [], fragment_prefix: "", fragment: "", **params) ⇒ Context
constructor
Create a completion context.
-
#tool ⇒ Toys::ToolDefinition
The tool being invoked, which should control the completion.
-
#with(**delta_params) ⇒ Toys::Completion::Context
Create a new completion context with the given modifications.
Constructor Details
#initialize(cli:, previous_words: [], fragment_prefix: "", fragment: "", **params) ⇒ Context
Create a completion context.
Extra params are optional and can be used to affect the behavior in specific cases. Currently these are:
shell: :bashcauses the FileSystem completion to include glob expansions, since bash doesn't handle those directlydisable_flags: truecauses tool completion to omit flag completions which is used by thetoys dobuilt-in tool
38 39 40 41 42 43 44 45 46 47 48 49 50 51 |
# File 'lib/toys/completion.rb', line 38 def initialize(cli:, previous_words: [], fragment_prefix: "", fragment: "", **params) @cli = cli @previous_words = previous_words @fragment_prefix = fragment_prefix @fragment = fragment extra_params = { cli: cli, previous_words: previous_words, fragment_prefix: fragment_prefix, fragment: fragment } @params = params.merge(extra_params) @tool = nil @args = nil @arg_parser = nil end |
Instance Attribute Details
#cli ⇒ Toys::CLI (readonly)
The CLI being run.
67 68 69 |
# File 'lib/toys/completion.rb', line 67 def cli @cli end |
#fragment ⇒ String (readonly)
The current string fragment to complete
85 86 87 |
# File 'lib/toys/completion.rb', line 85 def fragment @fragment end |
#fragment_prefix ⇒ String (readonly)
A non-completed prefix for the current fragment.
79 80 81 |
# File 'lib/toys/completion.rb', line 79 def fragment_prefix @fragment_prefix end |
#previous_words ⇒ Array<String> (readonly)
All previous words.
73 74 75 |
# File 'lib/toys/completion.rb', line 73 def previous_words @previous_words end |
Instance Method Details
#[](key) ⇒ Object Also known as: get
Get data for arbitrary key.
92 93 94 |
# File 'lib/toys/completion.rb', line 92 def [](key) @params[key] end |
#arg_parser ⇒ Toys::ArgParser
Current ArgParser indicating the status of argument parsing up to this point.
122 123 124 125 |
# File 'lib/toys/completion.rb', line 122 def arg_parser lookup_tool @arg_parser ||= ArgParser.new(@cli, @tool).parse(@args) end |
#args ⇒ Array<String>
An array of complete arguments passed to the tool, prior to the fragment to complete.
111 112 113 114 |
# File 'lib/toys/completion.rb', line 111 def args lookup_tool @args end |
#tool ⇒ Toys::ToolDefinition
The tool being invoked, which should control the completion.
101 102 103 104 |
# File 'lib/toys/completion.rb', line 101 def tool lookup_tool @tool end |
#with(**delta_params) ⇒ Toys::Completion::Context
Create a new completion context with the given modifications.
59 60 61 |
# File 'lib/toys/completion.rb', line 59 def with(**delta_params) Context.new(**@params, **delta_params) end |