Class: Toys::FlagGroup::Base

Inherits:
Object
  • Object
show all
Defined in:
lib/toys/flag_group.rb

Overview

The base class of a FlagGroup, implementing everything except validation.

Direct Known Subclasses

AtLeastOne, AtMostOne, ExactlyOne, Optional, Required

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#descToys::WrappableString

The short description string.

When reading, this is always returned as a WrappableString.

When setting, the description may be provided as any of the following:

  • A WrappableString.
  • A normal String, which will be transformed into a WrappableString using spaces as word delimiters.
  • An Array of String, which will be transformed into a WrappableString where each array element represents an individual word for wrapping.


86
87
88
# File 'lib/toys/flag_group.rb', line 86

def desc
  @desc
end

#flagsArray<Toys::Flag> (readonly)

An array of flags that are in this group. Do not modify the returned array.

Returns:



112
113
114
# File 'lib/toys/flag_group.rb', line 112

def flags
  @flags
end

#long_descArray<Toys::WrappableString>

The long description strings.

When reading, this is returned as an Array of WrappableString representing the lines in the description.

When setting, the description must be provided as an Array where each element may be any of the following:

  • A WrappableString representing one line.
  • A normal String representing a line. This will be transformed into a WrappableString using spaces as word delimiters.
  • An Array of String representing a line. This will be transformed into a WrappableString where each array element represents an individual word for wrapping.

Returns:



105
106
107
# File 'lib/toys/flag_group.rb', line 105

def long_desc
  @long_desc
end

#nameString, ... (readonly)

The symbolic name for this group

Returns:

  • (String, Symbol, nil)


69
70
71
# File 'lib/toys/flag_group.rb', line 69

def name
  @name
end

Instance Method Details

#append_long_desc(long_desc) ⇒ self

Append long description strings.

You must pass an array of lines in the long description. See #long_desc for details on how each line may be represented.

Parameters:

Returns:

  • (self)


163
164
165
166
# File 'lib/toys/flag_group.rb', line 163

def append_long_desc(long_desc)
  @long_desc.concat(WrappableString.make_array(long_desc))
  self
end

#empty?boolean

Returns true if this group is empty

Returns:

  • (boolean)


118
119
120
# File 'lib/toys/flag_group.rb', line 118

def empty?
  flags.empty?
end

#summaryString

Returns a string summarizing this group. This is generally either the short description or a representation of all the flags included.

Returns:

  • (String)


127
128
129
130
# File 'lib/toys/flag_group.rb', line 127

def summary
  return desc.to_s.inspect unless desc.empty?
  flags.map(&:display_name).inspect
end