Exception: Toys::Utils::GitCache::Error

Inherits:
StandardError
  • Object
show all
Defined in:
lib/toys/utils/git_cache.rb

Overview

Toys::Utils::GitCache encountered a failure

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(message, result) ⇒ Error

Create a Toys::Utils::GitCache::Error.

Parameters:

  • message (String)

    The error message

  • result (::Toys::Utils::Exec::Result)

    The result of a git command execution, or nil if this error was not due to a git command error.



450
451
452
453
# File 'lib/toys/utils/git_cache.rb', line 450

def initialize(message, result)
  super(message)
  @exec_result = result
end

Instance Attribute Details

#exec_result::Toys::Utils::Exec::Result (readonly)

Returns The result of a git command execution, or nil if this error was not due to a git command error.

Returns:



460
461
462
# File 'lib/toys/utils/git_cache.rb', line 460

def exec_result
  @exec_result
end